from typing import Optional, List
from pydantic import BaseModel, Field,  HttpUrl
from enum import Enum



class OrgRole(str, Enum):
    admin   = 'admin'
    member  = 'member'
    service = 'service'
    view    = 'view'
    

class OrgPostRequest(BaseModel):
    name:        str           = Field(min_length=1, max_length=39, description='Unique name for this org.')
    description: Optional[str] = Field(default=None, description='Optional user supplied description.')

class OrgPatchRequest(BaseModel):
    name:        Optional[str] = Field(max_length=39, description='Unique name for this org.')
    description: Optional[str] = Field(max_length=255, description='Optional user supplied description.')

class OrgMemberPostRequest(BaseModel):
    email:  str          = Field(description='The user_id of a member to invite to the org.')
    role:   OrgRole      = Field(description='The users role in the org')

class OrgMember(BaseModel):
    id:                  int      = Field(description="Unique membership id")
    name:                str      = Field(description="Member name")
    email:               str      = Field(description="Member email")    
    created_at:          float    = Field(description='The epoch timestamp when the membership was created.')
    updated_at:          float    = Field(description='The epoch timestamp when the membership was updated.')
    invited_by_name:     str      = Field(description="The name that invited this member to the org.")
    role:                OrgRole  = Field(description="The user's role in the org")
    accepted:            bool     = Field(description='True if the user has accepted the org membership.')
    


class Org(BaseModel):
    id:                  int           = Field(escription="Internal org id")
    name:                str           = Field(max_length=39, description='Unique name for this org.')
    description:         Optional[str] = Field(max_length=255, description='Optional user supplied description.')
    created_at:          float         = Field(description='The epoch timestamp when the org was created.')
    updated_at:          float         = Field(description='The epoch timestamp when the org was updated.')
    created_by:          Optional[int] = Field(description='The user_id of the user that created the org.')
    subscription_id:     str           = Field(description='The subscription_id for the org.')    
    gpt4_credits:        int           = Field(description='The number of GPT-4 message credits currently available to the org users for chat.jiggy.ai.')
    gpt3_5_credits:      int           = Field(description='The number of GPT-3 message credits currently available to the org users for chat.jiggy.ai.')
    subscription_status: str           = Field(description='The stripe subscription status.')
     



from typing import List
from .org import Org
from .collection import Collection
from .models.user import User, ApiKey
from .jiggybase_session import JiggyBaseSession
from .models.chat import Message, TypedCompletionRequest
from pydantic import BaseModel    

class JiggyBase():

    def __init__(self, api_key=None):
        self.session = JiggyBaseSession(api_key=api_key)     
        
    def orgs(self) -> List[Org]:
        """
        return all Orgs that the user is a member of
        """
        resp = self.session.get('/orgs')
        return [Org(self.session, **i) for i in resp.json()]

    def get_org(self, name_or_id) -> Org:
        """
        get org by name or id
        raises Exception if an exact match for name is not found
        """
        orgs = [t for t in self.session.get('/orgs').json() if t['name'] == name_or_id or t['id'] == name_or_id]
        if len(orgs):
            return Org(self.session, **orgs[0])
        raise Exception(f'Org "{name_or_id}" not found')


    def api_keys(self) -> List[ApiKey]:
        """
        return user's api keys
        """
        return [ApiKey(**i) for i in self.session.get('/apikey').json()]


    def authenticated_user(self) -> User:
        """
        return the authenticated user's User object
        """
        return User(**self.session.get("/users/current").json())


    def create_org(self, name : str) -> Org:
        """
        create an Org
        """
        resp = self.session.post("/orgs", json={"name":name})
        return Org(self.session, **resp.json())

    def collections(self) -> List[Collection]:
        """
        return all Collections in all Orgs that the user is a member of
        """
        resp = self.session.get("/collections")
        return [Collection(self.session, **c) for c in resp.json()]
   
    def collection_names(self) -> List[Collection]:
        """
        return the collection display names for all collections in all Orgs that the user is a member of
        """
        return [c.disply_name for c in self.collections()]

    def collection_hostnames(self) -> List[Collection]:
        """
        return the unique collection hostname for all collections in all Orgs that the user is a member of
        """
        return [c.hostname for c in self.collections()]

            
    def collection(self, name : str) -> Collection:        
        """
        return a collection of the specified display name or hostname
        """
        for collection in self.collections():
            if collection.hostname == name or collection.display_name.lower() == name.lower():
                return collection
        raise ValueError(f'Collection "{name}" not found')


    def _extract_typed_completion(self,
                                 messages       : List[Message],
                                 pydantic_model : BaseModel,
                                 temperature    : float = 0,
                                 model          : str   = 'gpt-3.5-turbo') -> BaseModel:
        """
        lower-level interface to the extract_typed_completion endpoint
        """
        tcr = TypedCompletionRequest(model       = model, 
                                     messages    = messages,
                                     json_schema = pydantic_model.schema_json(),
                                     temperature = temperature,
                                     stream      = False)
        
        rsp = self.session.post("/extract/typed_completions", model=tcr)
        return pydantic_model.parse_obj(rsp.json())


    def  extract_typed_completion(self,
                                  content       : str,
                                  pydantic_model : BaseModel, 
                                  temperature    : float = 0,
                                  model          : str   = 'gpt-3.5-turbo') -> BaseModel:
        """
        Higher-level interface to structured extraction from unstructured content.
        Just provide the unstructured text and the pydantic model to extract.
        """
        # Set up messages
        messages = [{"role": "user", "content": f"Extract the {pydantic_model.__name__} information from the following content:"},
                    {"role": "user", "content": content}]

        return self._extract_typed_completion(messages, pydantic_model, temperature, model)



import os
from typing import Optional, List, Iterator, Tuple
from pydantic import BaseModel, Field, BaseConfig, HttpUrl
from enum import Enum
from .models import collection, CollectionChatConfig, PatchCollectionChatConfig
from .jiggybase_session import JiggyBaseSession
from .models import UpsertResponse,  Query, QueryRequest, QueryResponse, UpsertRequest, Document, DocumentChunk, DeleteRequest, DeleteResponse, DocumentMetadataFilter, DocChunksResponse
from .models import Message, CompletionRequest, ChatCompletion 
from .chat_stream import extract_content_from_sse_bytes
from .models import ExtractMetadataConfig
from .models import CollectionPatchRequest, PluginAuthConfigOAuth, PatchPluginOAuthConfigRequest
from .models import DocumentMetadata
from typing import Union, List
from urllib.parse import quote_plus


        
class Collection(collection.Collection):
    """
    derived from models.collection.Collection data model for purposes of adding management methods
    """
    class Config(BaseConfig):
        extra = "allow"

    def __init__(self, session, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.session = session        
        api_key = self.session.api_key
        self.plugin_session = JiggyBaseSession(host=f'https://{kwargs["fqdn"]}', api='', api_key=api_key)
        self.chat_session = JiggyBaseSession(host=session.host, api='v1', api_key=api_key)
        
    def set_description(self, description:str) -> "Collection":
        """
        Update an existing collection using its ID and the provided description.
        """
        patch_request = CollectionPatchRequest(description=description)
        rsp = self.session.patch(f"/orgs/{self.org_id}/collections/{self.id}", model=patch_request)
        self.description = rsp.json()['description']


    def set_oauth_verification_token(self, openai_verification_token: str) -> PluginAuthConfigOAuth:
        """
        Set the OpenAI verification token for this collection's plugin.
        This is the token that OpenAI provides during while registering a plugin configured to use Oauth.
        """
        patch_request = PatchPluginOAuthConfigRequest(openai_verification_token=openai_verification_token)
        rsp = self.session.patch(f"/orgs/{self.org_id}/collections/{self.id}/plugin_oauth", model=patch_request)
        return PluginAuthConfigOAuth(**rsp.json())

    def plugin_oauth_config(self) -> PluginAuthConfigOAuth:
        """
        Get the OAuth configuration for this collection's plugin.
        The client_id and client_secret returned here are configured with OpenAI while 
        registering a plugin configured to use Oauth.
        """
        rsp = self.session.get(f"/orgs/{self.org_id}/collections/{self.id}/plugin_oauth")
        return PluginAuthConfigOAuth(**rsp.json())

    def delete(self):
        """
        Delete a collection.  Warning: this is permanent.
        """
        self.session.delete(f"/orgs/{self.org_id}/collections/{self.id}")


    def get_chat_config(self) -> CollectionChatConfig:
        """
        Get the chat configuration for this collection.
        """
        rsp = self.session.get(f"/orgs/{self.org_id}/collections/{self.id}/chat_config")
        return collection.CollectionChatConfig(**rsp.json())

    def update_chat_config(self, model: str, prompt_task_id: int) -> CollectionChatConfig:
        """
        Update the chat configuration for this collection.
        """
        rsp = self.session.patch(f"/orgs/{self.org_id}/collections/{self.id}/chat_config/{model}", model=PatchCollectionChatConfig(prompt_task_id=prompt_task_id))
        return CollectionChatConfig(**rsp.json())

 
    def upsert_file(self, file_path: str, mimetype: str = None, id: str = None, metadata : DocumentMetadata = None) -> UpsertResponse:
        """
        Add a file to the collection.
        Mimetype can be specified, otherwise it will be inferred from the file extension.
        The doc id can be specified, otherwise it will be generated.
        Metadata can be specified, otherwise some metadata will be inferred from the file.
        """
        if not os.path.exists(file_path):
            raise ValueError("File not found")
        with open(file_path, "rb") as file:
            files = {"file": (os.path.basename(file_path), file, mimetype)}
            params = {'id': id} if id else {}
            if metadata:
                rsp = self.plugin_session.post("/upsert-file", 
                                               files=files, 
                                               params=params,
                                               data={'metadata': metadata.json(exclude_unset=True)})
            else:
                rsp = self.plugin_session.post("/upsert-file", params=params, files=files)
        return UpsertResponse.parse_obj(rsp.json())

    def upsert(self, documents: List[Document]) -> UpsertResponse:
        """
        Add a list of Document objects to the collection.
        """
        upsert_request = UpsertRequest(documents=documents)
        rsp = self.plugin_session.post("/upsert", model=upsert_request)
        return  UpsertResponse.parse_obj(rsp.json())
    
    def query(self, queries: Union[str, List[str], Query], top_k : int = 10) -> QueryResponse:
        """
        Query the collection returning the top_k results for each query.
        queries can be either a single string, a list of strings, or a list of Query objects.
        if it is a string or list of strings, the specified top_k will be used for each of the queries.
        Returns a QueryResponse object.
        """
        if isinstance(queries, str):
            queries = [Query(query=queries, top_k=top_k)]
        elif isinstance(queries, Query):
            queries = [Query(query=q, top_k=top_k) for q in queries]
        qr = QueryRequest(queries=queries)
        rsp = self.plugin_session.post("/query", model=qr)
        return  QueryResponse.parse_obj(rsp.json())

    def get_doc(self, id: str) -> list[DocumentChunk]:
        """
        Get a document by id
        """
        encoded_id = quote_plus(id)
        rsp = self.plugin_session.get(f"/documents?doc_id={encoded_id}")
        return [DocumentChunk.parse_obj(c) for c in rsp.json()]
    
    def get_chunks(self, 
                   start: int = 0, 
                   limit: int = 10, 
                   reverse: bool = True) -> List[DocumentChunk]:
        """
        low level interface for iterating through the chunks in a collection
        start - Offset of the first result to return
        limit - Number of results to return starting from the offset
        reverse - Reverse the order of the items returned
        """
        params = {"start": start, "limit": limit, "reverse": reverse}
        rsp = self.plugin_session.get("/chunks", params=params)
        return [DocumentChunk.parse_obj(chunk) for chunk in rsp.json()]

    def get_doc_chunks(self, 
                       index: int = -1, 
                       limit: int = 10, 
                       reverse: bool = True,
                       max_chunks_per_doc = 1) -> Tuple[List[List[DocumentChunk]], int]:
         """
         low level interface for iterating through the initial chunks for all docs in a collection
         index - index of the first result to return, should be -1 to start at the end
         limit - Number of results to return starting from the offset
         reverse - Reverse the order of the items returned; True to return newest first
         max_chunks_per_doc - maximum number of chunks to return for each document
         """
         params = {"index": index, "limit": limit, "reverse": reverse, "max_chunks_per_doc": max_chunks_per_doc}
         rsp = self.plugin_session.get("/doc_chunks", params=params)
         dcr_rsp = DocChunksResponse.parse_obj(rsp.json())
         return dcr_rsp.docs, dcr_rsp.next_index

    def delete_docs(self, 
                    ids                      : Optional[List[str]] = None, 
                    document_metadata_filter : Optional[DocumentMetadataFilter] = None, 
                    delete_all               : Optional[bool] = False) -> DeleteResponse:
        """
        Delete items in the collection by document id's or document metadata filter.
        A delete_all option is also provided to delete all documents in the collection.
        """
        delete_request = DeleteRequest(ids=ids, filter=document_metadata_filter, delete_all=delete_all)
        rsp = self.plugin_session.delete("/delete", model=delete_request)
        return DeleteResponse.parse_obj(rsp.json())


    def _chat_completion(self, 
                         messages: List[Message],
                         model = 'gpt-3.5-turbo',
                         max_tokens = None,
                         temperature = .1) -> ChatCompletion:
        """
        low level interface for chat completion
        The input here mirrors openai chat completion parameters, while the output is different
        """
        cr = CompletionRequest(model       = f'{self.name}_{model}', 
                               messages    = messages,
                               max_tokens  = max_tokens,
                               temperature = temperature,
                               stream      = False)
        rsp = self.chat_session.post("/chat/completions", model=cr)          
        return ChatCompletion.parse_obj(rsp.json())

    def _chat_completion_stream_str(self, 
                         messages: List[Message],
                         model = 'gpt-3.5-turbo',
                         max_tokens = None,
                         temperature = .1) -> Iterator[str]:
        """
        low level interface for chat completion with streaming
        yields the model output as an iteration of strings
        """
        cr = CompletionRequest(model       = f'{self.name}_{model}', 
                               messages    = messages,
                               max_tokens  = max_tokens,
                               temperature = temperature,
                               stream      = True)
        rsp = self.chat_session.post("/chat/completions", model=cr, stream=True)
        for line in rsp.iter_lines():
            if line:  # filter out keep-alive newlines        
                yield extract_content_from_sse_bytes(line)

    def get_extract_metadata_config(self):
        """
        Get the metadata configuration for this collection.
        """
        rsp = self.session.get(f"/orgs/{self.org_id}/collections/{self.id}/extract_metadata_config")
        return ExtractMetadataConfig(**rsp.json())

    def patch_extract_metadata_config(self, config: ExtractMetadataConfig):
        """
        Get the metadata configuration for this collection.
        """
        rsp = self.session.patch(f"/orgs/{self.org_id}/collections/{self.id}/extract_metadata_config", model=config)
        return ExtractMetadataConfig(**rsp.json())    

    def __str__(self) -> str:
        return f"Collection(id={self.id:4}, name={self.display_name:30}, hostname={self.hostname:30}, org_id={self.org_id:4},\n" \
               f"           description='{self.description}')"

    def __repr__(self) -> str:
        return str(self)    


from .auth import *
from .user import *
from .org import *
from .embedding import *
from .metadata import *
from .chunk import *
from .chatmodel import *
#from .plugin_config import *
from .plugin import *
from .collection import *
from .prompt import *
from .chat import *



import jiggybase
from pydantic import BaseModel, Field
from typing import Optional, List
from datetime import date
from pydantic import BaseModel, Field, ValidationError, validator
from time import time

jb = jiggybase.JiggyBase()

gpt3_times =0 
gpt4_times = 0
count  = 0




# Define the pydantic model
class BookInformation(BaseModel):
    title: str = Field(description="The title of the book")
    author: str = Field(description="The name of the book's author")
    publication_year: Optional[int] = Field(description="The publication year of the book")
    genre: Optional[str] = Field(description="The genre of the book")
    characters: Optional[List[str]] = Field(description="A list of the main characters in the book")
    summary: Optional[str] = Field(min_length=10, description="A brief summary of the book's plot")

# Input unstructured text
unstructured_text = """
Pride and Prejudice is a novel by Jane Austen, published in 1813.
This classic novel follows the story of Elizabeth Bennet, the protagonist, as she navigates issues of manners, morality, education, and marriage in the society of the landed gentry of early 19th-century England.
The story revolves primarily around Elizabeth and her relationship with the haughty yet enigmatic Mr. Darcy.
The book is set in rural England, and it is notable for its wit and humor as well as its commentary on class distinctions, social norms, and values.
Some of the main characters in the novel include Elizabeth Bennet, Mr. Darcy, Jane Bennet, Mr. Bingley, Lydia Bennet, and Mr. Wickham.
Pride and Prejudice is considered a classic work of English literature and has been adapted numerous times for television, film, and stage.
It is often categorized as a romantic novel, but it also has elements of satire and social commentary.
"""

t0 = time()
model3 = jb.extract_typed_completion(unstructured_text, BookInformation, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, BookInformation, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)
count += 1





class DocumentMetadata(BaseModel):
    title      : Optional[str] = Field(description='The title of the content')
    author     : Optional[str] = Field(description='The author of the content')
    created_at : Optional[str] = Field(description='The date in the format YYYY-MM-DD that the content was created if it appears in the content')
    language   : str           = Field(description='The 2 character ISO 639-1 language code of the primary language of the content')


unstructured_text = """Updated 24th March 2023
Generative AI - Chapter 1: Establishing the Investment Framework
BlackLake Equity Research
The advent of cloud computing paved the way for new investment opportunities by facilitating the provision of software as a service. Generative AI takes this a step further, offering additional tools to boost end-user productivity. While traditional AI has proven useful in predicting outcomes, Generative AI specializes in creating content such as text, video, images, or computer code - a feat previously unachievable. Large Language Models (LLMs) play a crucial role as enablers of GAI, displaying an unprecedented level of expertise and intelligence. AI holds the potential to give rise to new enterprises and furnish existing players with fresh growth opportunities by greatly enhancing end-user productivity."""



t0 = time()
model3 = jb.extract_typed_completion(unstructured_text, DocumentMetadata, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, DocumentMetadata, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)
count += 1



# Define the pydantic models
class Course(BaseModel):
    title: str = Field(description="The title of the course")
    instructor: str = Field(description="The name of the course's instructor")
    duration_hours: Optional[int] = Field(description="The duration of the course in hours")

class Curriculum(BaseModel):
    curriculum_title: str = Field(description="The title of the curriculum")
    description: Optional[str] = Field(description="A brief description of the curriculum")
    courses: List[Course] = Field(description="A list of the courses included in the curriculum")

# Input unstructured text
unstructured_text = """
The Data Science Bootcamp is a comprehensive curriculum designed to provide students with the fundamental knowledge and practical skills required for a career in data science.
The bootcamp includes the following four courses:

1. Introduction to Python: This course, taught by John Doe, covers the basics of the Python programming language and its applications in data science. It lasts for 20 hours.

2. Data Wrangling and Visualization: Led by Jane Smith, this course teaches students how to clean, manipulate, and visualize data using popular Python libraries like pandas and Matplotlib. The course duration is 30 hours.

3. Machine Learning Foundations: In this 40-hour course, instructor Michael Brown introduces the core concepts and algorithms of machine learning, including supervised and unsupervised learning techniques.

4. Advanced Topics in Data Science: Taught by Sarah Johnson and lasting 35 hours, this course explores advanced data science techniques such as deep learning, natural language processing, and time series analysis.
"""

t0 = time()
model3 = jb.extract_typed_completion(unstructured_text, Curriculum, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, Curriculum, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)
count += 1




# Define the pydantic model
class Event(BaseModel):
    event_name: str = Field(description="The name of the event")
    start_date: date = Field(description="The start date of the event")
    end_date: date = Field(description="The end date of the event")

    @validator('end_date')
    def start_date_must_be_prior_to_end_date(cls, end_date, values):
        start_date = values.get('start_date')
        if start_date and end_date <= start_date:
            raise ValidationError(f"End date ({end_date}) must be after start date ({start_date})")
        return end_date

class EventList(BaseModel):
    events: List[Event] = Field(description="A list of events")

# Input unstructured text
unstructured_text = """
Calendar Update
Tuesday, April 27, 2023
We have a series of exciting events happening over the next few weeks!

1. Next Monday, the Annual AI Conference will kick off, ending on Wednesday. The event will focus on the latest advancements in artificial intelligence and machine learning, with discussions led by industry experts.

2. In two weeks, the three-day Startup Showcase will take place, where new tech startups will demonstrate their innovative products and services. The event aims to foster collaboration and investment opportunities.

3. The following week, we are hosting a Blockchain Summit, running from Tuesday to Thursday. The summit will explore the potential applications of blockchain technology across various industries, featuring engaging panel discussions and presentations.

4. Finally, at the end of the month, a four-day Virtual Reality Festival will commence. This event celebrates the rapid progress of virtual and augmented reality technologies, with immersive exhibits and experiences available for all attendees.

Make sure to mark your calendars for these thrilling events and seize the opportunity to learn more about the ever-evolving world of technology!
"""

t0 = time()

model3 = jb.extract_typed_completion(unstructured_text, EventList, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, EventList, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)
count += 1




# Define the pydantic model
class MovieInformation(BaseModel):
    title: str = Field(description="The title of the movie")
    director: str = Field(description="The name of the movie's director")
    release_year: Optional[int] = Field(description="The release year of the movie")
    cast: Optional[List[str]] = Field(description="A list of the main actors in the movie")
    genre: Optional[List[str]] = Field(description="The genres of the movie")
    duration_minutes: Optional[int] = Field(description="The duration of the movie in minutes")
    plot: Optional[str] = Field(min_length=10, description="A brief summary of the movie's plot")
    awards: Optional[List[str]] = Field(description="A list of the awards won by the movie")

# Input unstructured text
unstructured_text = """
The Godfather is a 1972 American crime film directed by Francis Ford Coppola, who co-wrote the screenplay with Mario Puzo, based on Puzo's best-selling 1969 novel of the same name.
The film stars Marlon Brando, Al Pacino, and James Caan, and tells the story of the powerful Italian-American crime family of Don Vito Corleone (played by Brando).
When an organized crime dynasty's aging patriarch transfers control of his clandestine empire to his reluctant son, Michael Corleone (played by Pacino), a series of events unfold that threatens the stability of the family and the entire criminal underworld.
Set in New York City and spanning the years 1945 to 1955, The Godfather is notable for its realistic depiction of the Mafia and its exploration of themes such as power, loyalty, betrayal, and family dynamics.
The film was met with widespread critical acclaim upon release and is commonly regarded as one of the greatest films in world cinema.
It won three Academy Awards, including Best Picture, Best Actor (Marlon Brando), and Best Adapted Screenplay, and has been selected for preservation in the United States National Film Registry.
The Godfather is a crime drama film and is widely regarded as one of the most influential films in the gangster genre.
Its running time is approximately 175 minutes.
"""



t0 = time()
model3 = jb.extract_typed_completion(unstructured_text, MovieInformation, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, MovieInformation, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)
count += 1



    
# The data class we wish to extract from the unstructured text
class UserDetails(BaseModel):
    name: str = Field(description="The name of the user")
    age: Optional[int] = Field(description="The age of the user")
    email: Optional[str] = Field(description="The email address of the user")
    country: str = Field(description="The country the user resides in")

# Example unstructured text from which to extract the UserDetails
unstructured_text = """
John Doe, a software engineer from San Francisco, California, started his career in the tech industry in the United States in 2010.
He initially worked for a small startup before joining a larger multinational company in 2014.
John, currently 30 years old, is passionate about programming and solving complex problems.
He has attended multiple conferences across the world and has collaborated with colleagues from various countries on numerous projects.
Despite his busy work life, John finds time for his hobbies, such as photography and hiking.
He enjoys traveling to different countries, and one of his favorite trips was to Japan in 2019.
Do you need to get in touch with John? Feel free to reach out to him at john.doe@example.com.
"""

t0 = time()
model3 = jb.extract_typed_completion(unstructured_text, UserDetails, temperature=0, model='gpt-3.5-turbo')
gpt3_times += time() - t0
print(model3)

t0 = time()
model4 = jb.extract_typed_completion(unstructured_text, UserDetails, temperature=0, model='gpt-4')
gpt4_times += time() - t0
print(model4)    
count += 1


print(f"GPT-3.5 Turbo: {gpt3_times/count} s")
print(f"GPT-4:         {gpt4_times/count} s")


import jiggybase

collection = jiggybase.JiggyBase().collection('hackernews-summary')   # replace with your collection name

messages = [{'role':'user',  'content': 'articles about python 3.11'}]

rsp = collection._chat_completion(messages, temperature=0, model="gpt-3.5-turbo")   # note _ as this is preliminary low-level interface

print(rsp)

"""
Here are some articles about Python 3.11:

1. Python 3.11.1, 3.10.9, 3.9.16, 3.8.16, 3.7.16, and 3.12.0 alpha 3 now available - This is a blog post from the official Python Software Foundation blog. The post is announcing the release of Python 3.11.1, 3.10.9, 3.9.16, 3.8.16, 3.7.16, and 3.12.0 alpha 3. The post goes over some of the security content in the new releases.

2. Python 3.11 Is Much Faster, but Pyston and PyPy Still Show Advantages - This is a news article from the website www.phoronix.com. It discusses the recent Python 3.11 beta benchmarks and compares them to alternative Python implementations like PyPy and Pyston. It also includes Python 3.11b3 results.

3. A Team at Microsoft Is Helping Make Python Faster - This is a blog post about the Faster CPython Team at Microsoft. It details the team's mission to make Python faster, the specialized knowledge and collaboration of the team, and Microsoft's commitment to the Python community. It also highlights the team's work on Python 3.11 and their plans for Python 3.12.

I hope this helps!
"""



import os
from pydantic import BaseModel, HttpUrl, Field, field_validator
from typing import List, Optional
from enum import Enum
from .plugin import PluginAuthType        
import re

MODELNAME = re.compile("[a-zA-Z][a-zA-Z0-9_]*")

class PluginConfig(BaseModel): 
    """
    The user-customizable part of PluginConfig
    """
    name_for_model:        str = Field("retrieval", max_length=50, description="The name of the plugin as it will be appear to the model.")
    name_for_human:        str = Field(max_length=50, description="The name of the plugin as it will be appear to the human user.")
    description_for_model: str = Field(max_length=4000, description="Description of the plugin as it will appear to the model.")
    description_for_human: str = Field(max_length=120, description="Description of the plugin as it will appear to the human user.")
    logo:                  Optional[str] = Field(description="The logo for the plugin")
    logo_url:              Optional[HttpUrl] = Field(description="The logo url for the plugin")

    @field_validator('name_for_model')
    def _name_for_model(cls, v):
        if len(v) > 50 or not v:
            raise ValueError(f'"{v}" is an invalid model name. It must not be empty and is limited to 50 characters.')
        elif not bool(MODELNAME.match(v)):
         raise ValueError(f'"{v}" is an invalid model name. It can only contain letters, numbers, and underscores, and must start with a letter.')
        return v


class PatchPluginConfigRequest(BaseModel):
    name_for_model:        Optional[str] = Field(description="The plugin name for the model")
    name_for_human:        Optional[str] = Field(description="The plugin name for human")
    description_for_model: Optional[str] = Field(description="The plugin description for the model")
    description_for_human: Optional[str] = Field(description="The plugin description for human")
    logo:                  Optional[str] = Field(description="The logo for the plugin")
    logo_url:              Optional[HttpUrl] = Field(description="The logo url for the plugin")

    @field_validator('name_for_model')
    def _name_for_model(cls, v):
        if len(v) > 50 or not v:
            raise ValueError(f'"{v}" is an invalid model name. It must not be empty and is limited to 50 characters.')
        elif not bool(MODELNAME.match(v)):
         raise ValueError(f'"{v}" is an invalid model name. It can only contain letters, numbers, and underscores, and must start with a letter.')
        return v



class OpenAIVerificationToken(BaseModel):
    openai: str


class OpenAIPluginAuthConfigOAuth(BaseModel):
    """
    The Plugin Oauth configuration as it is presented in the ai-plugin.json 
    """
    type:                       str         = Field("oauth", const=True)
    client_url:                 HttpUrl     = Field(description="ChatGPT will direct the user’s browser to this url to log in to the plugin")
    authorization_url:          HttpUrl     = Field(description="After successful login ChatGPT will complete the OAuth flow by making a POST request to this URL")
    scope:                      str         = Field(description="The scope used for the OAuth flow")    
    authorization_content_type: str         = Field("application/json", const=True)
    verification_tokens:        OpenAIVerificationToken = Field(description="The verification token to send to OpenAI for the plugin")




import csv
import os
import tempfile
import shutil
from dotenv import load_dotenv
import jiggybase

def get_answer(question):
    messages = [{'role':'user',  'content': question}]
    collection_name = os.getenv("JIGGYBASE_COLLECTION_NAME")
    collection = jiggybase.JiggyBase().collection(collection_name)
    rsp = collection._chat_completion(messages)
    return rsp

def fill_answers(input_csv):
    with tempfile.NamedTemporaryFile(mode='w+', newline='', delete=False) as temp_file:
        with open(input_csv, "r", newline='') as infile, open(temp_file.name, "w", newline='') as outfile:
            reader = csv.reader(infile)
            writer = csv.writer(outfile)
    
            # Verify the header and write it to the output file
            header = next(reader)
            assert header == ["question", "answer"], "Invalid header"
            writer.writerow(header)
    
            # Iterate over the rows, filling in answers when needed
            for row in reader:
                question, answer = row
                if not answer or answer.strip() == "":
                    answer = get_answer(question)
                    row = [question, answer]
                writer.writerow(row)

        # Replace the original input file with the updated file
        shutil.move(temp_file.name, input_csv)

load_dotenv()
input_csv = "input.csv"
fill_answers(input_csv)



import os
import json
import jiggybase
from  jiggybase.models import Document, DocumentMetadata, Source


jb = jiggybase.JiggyBase()

JIGGYBASE_ORG = os.environ["JIGGYBASE_ORG"]
JIGGYBASE_COLLECTION = os.environ["JIGGYBASE_COLLECTION"]

org = jb.get_org(JIGGYBASE_ORG)
collection = org.collection(JIGGYBASE_COLLECTION)


other_metadata = {"to": ['bob@example.com'], "cc": ['hr@example.com', 'legal@example.com']}

# Example email metadata
email_metadata = DocumentMetadata(
    source     = Source.email,
    created_at = "2022-02-15T10:30:00",        # using ISO 8601 format (YYYY-MM-DDTHH:MM:SS) makes it possible to filter by date
    author     = "Alice <alice@example.com>",  # sender name and optional email address
    title      = "Reminder: Friday Meeting",   # subject
    description = json.dumps(other_metadata)   # put any other metadata here that is associated with the entire document 
)

# Create a Document object with the email text and metadata
email_document = Document(
    id       = "XX4HJ823930JK",   # unique ID for the document
    metadata = email_metadata,
    text     = "Hello Bob, I hope you're doing well. I just wanted to remind you about the upcoming meeting on Friday. Best, Alice",    
)

# Upsert the email_document into test_collection
upsert_response = collection.upsert(documents=[email_document])
print(upsert_response)



#!/usr/bin/env python3

import jiggybase
try:
    import IPython
except:
    raise Exception("'pip install IPython' run this command first")

def main():
    
    print("\nStarting JiggyBase IPython environment\n")

    jb = jiggybase.JiggyBase()
    orgs = jb.orgs()
    collections = jb.collections()
    
    print(f"'jb'          set to jiggybase.JiggyBase() instance")
    print(f"'orgs'        set to list of your orgs")
    print(f"'collections' set to list of your collections")    
    print()    
    IPython.start_ipython(argv=[], user_ns={"jb": jb, "orgs": orgs, "collections": collections})
    

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
Specification:

Sync the text content in a Confluence Space into a JiggyBase Collection


Takes the following config from the command line or via environment variables:

--user  or ATLASSIAN_USER   environment variable
--token or ATLASSIAN_TOKEN  environment variable   
--url   or CONFLUENCE_URL   environment variable   
--space or CONFLUENCE_SPACE environment variable   The space key, id, or name of the space to sync
--org   or JIGGYBASE_ORG    environment variable   The JiggyBase org to sync to


To create an Atlassian API token go to 
https://id.atlassian.com/manage-profile/security/api-tokens
and click on Create API token. Save the value into an environment variable.
E.g. on Macos
export ATLASSIAN_TOKEN=XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


If a JiggyBase API token is not set then the JiggyBase SDK will open a web browser on the page to copy it from the JiggyBase dashboard.

Implementation notes:
Uses page['version']['when'] as document created_at timestamp and version indicator
Uses page['id'] as document id (unique within a collection)

If a given document ID exists in both confluence and jiggybase, check  
Re-uploads a page if the version has changed from what is in the metadata

"""

# pip install atlassian-python-api
from atlassian import Confluence
import os
import argparse
import jiggybase
import requests 


def upsert_file(collection : jiggybase.models.Collection, filename : str):
    print(f'Uploading {filename}')
    try:
        upsert_rsp = collection.upsert_file(filename)
    except Exception as e:            
        print(f'error on {filename}: {e}')
        return
    doc_id = upsert_rsp.ids[0]
    dcl =  collection.get_doc(doc_id)
    text_len = len(" ".join([dc.text for dc in dcl]))
    title = dcl[0].metadata.title if dcl[0].metadata.title else "Unnown Title"
    print(f'Processed {filename}: "{title}"  {text_len//1024} KB text ({len(dcl)} chunks)')


def save_attachment_to_file(confluence, attachment):
    title = attachment['title']
    download_link = attachment['_links']['download']
    download_url = f'{confluence.url}{download_link}'
    
    response = requests.get(download_url, auth=(confluence.username, confluence.password))
    
    if response.status_code == 200:
        with open(title, 'wb') as file:
            file.write(response.content)
        return title
    else:
        print(f"Error downloading attachment '{title}': {response.status_code}")
        return None



def sync_confluence_space_to_jiggybase_collection(confluence, space, org):
    """
    """
    try:
        space_collection = org.collection(space['name'])
        print(f"Found existing JiggyBase collection {space['name']}")
    except:
        space_collection = org.create_collection(space['name'])
        print(f"Created new JiggyBase collection {space['name']}")

    # Delete all documents in the collection
    print(f"Deleting all prior documents in JiggyBase collection {space['name']}")
    space_collection.delete_docs(delete_all=True)

    # Get all pages in the space
    pages = confluence.get_all_pages_from_space(space['key'])

    # Loop through each page and process it
    for page in pages:
        # Download page as Word document
        doc_content = confluence.get_page_as_word(page['id'])
        doc_filename = f"{page['title']}.doc"
        
        # Save the Word document locally
        with open(doc_filename, 'wb') as doc_file:
            doc_file.write(doc_content)
        
        # Upload the Word document to JiggyBase collection
        upsert_file(space_collection, doc_filename)
        os.remove(doc_filename)
        
        # Get all attachments from the page
        attachments = confluence.get_attachments_from_content(page['id'])['results']

        # Download and upload each attachment to JiggyBase collection
        for attachment in attachments:
            if attachment['title'].split('.')[-1] in ['svg']:
                # skip unsupported file types
                continue
            print(f"Downloading attachment {attachment['title']} from page {page['title']}")
            attachment_filename = save_attachment_to_file(confluence, attachment)
            if attachment_filename:
                upsert_file(space_collection, attachment_filename)            
            os.remove(attachment_filename)
                
        print(f"Finished syncing {page['title']} and its attachments.")


        

def main():
    # Handle command line arguments and environment variables
    parser = argparse.ArgumentParser(description="Sync Confluence Space to JiggyBase Collection")
    parser.add_argument('--user', default=os.environ.get('ATLASSIAN_USER'))
    parser.add_argument('--token', default=os.environ.get('ATLASSIAN_TOKEN'))
    parser.add_argument('--url', default=os.environ.get('CONFLUENCE_URL'))
    parser.add_argument('--space', default=os.environ.get('CONFLUENCE_SPACE'))
    parser.add_argument('--org', default=os.environ.get('JIGGYBASE_ORG'))
    
    args = parser.parse_args()

    jb = jiggybase.JiggyBase()

    orgs = jb.orgs()
        
   # Check if all required arguments are provided
    missing_args = []
    if not args.user:
        missing_args.append('--user  or ATLASSIAN_USER   environment variable')
    if not args.token:
        missing_args.append('--token or ATLASSIAN_TOKEN  environment variable')
    if not args.url:
        missing_args.append('--url   or CONFLUENCE_URL   environment variable')
    if not args.org and len(orgs) > 1:
        missing_args.append('--org   or JIGGYBASE_ORG   environment variable')
        
    if missing_args:
        print("\nError: The following required arguments are missing:")
        for arg in missing_args:
            print(f"- {arg}")
        print()
        parser.print_help()
        return
    
    # Connect to Confluence API
    confluence = Confluence(args.url, username=args.user, password=args.token)
    
    # Get all spaces and find the matching space
    all_spaces = confluence.get_all_spaces()
    matching_space = None
    
    for space in all_spaces['results']:
        if args.space in [space['key'], space['name'], str(space['id'])]:
            matching_space = space
            break

    if matching_space:
        print(f"Matched Space: {matching_space['name']} (ID: {matching_space['id']})")
    else:
        print("No matching space found. Available spaces:")
        for space in all_spaces['results']:
            print(f"- {space['name']:20}  (ID: {space['id']})  key = {space['key']}")
        return
    
    if len(orgs) == 1:
        org = orgs[0]
    else:
        org = jb.get_org(args.org)
    print(f"Using JiggyBase org {org.name} (ID: {org.id})")
    
    sync_confluence_space_to_jiggybase_collection(confluence, matching_space, org)

if __name__ == "__main__":
    main()




from typing import Optional
from pydantic import BaseModel, Field


        
class CollectionPostRequest(BaseModel):
    """
    Used to create a new Collection service
    """
    display_name: str                      = Field(description="The human friendly display name for the collection.")
    description:  Optional[str]            = Field(description="A description of the collection.")


class CollectionPatchRequest(BaseModel):
    """
    Used to modify an existing service
    """
    description:  Optional[str] = Field(description="A description of the collection.")
    display_name: Optional[str] = Field(description="The human friendly display name for the collection.")





class Collection(BaseModel):
    """
    A managed collection of searchable documents exposed via as a ChatGPT plugin and a REST API.
    """
    id:           int           = Field(description="The unique ID of the service")
    display_name: str           = Field(description="The human friendly display name for the collection.")    
    description:  Optional[str] = Field(description="A description of the collection.")
    hostname:     str           = Field(description="The unique hostname for the collection service. The hostname part of the fqdn")
    fqdn:         str           = Field(description="The FQDN for the collection service")
    org_id:       int           = Field(description='The Org that owns this Service.')
    created_by:   int           = Field(description='The user_id that created this item.')
    updated_by:   int           = Field(description='The user_id that last modified this item.')
    created_at:   float         = Field(description='The epoch timestamp when the collection was created.')
    updated_at:   float         = Field(description='The epoch timestamp when the collection was created.')




from typing import Optional, Union, List
from pydantic import BaseModel, Field
from enum import Enum
from pydantic import BaseModel, Field, HttpUrl




class PluginAuthType(str, Enum):
    """
    Note that this is for custom plugins available in the JiggyBase enterprise tier and unrelated to the main JiggyBase plugin in the OpenAI plugin store.
    """
    bearer :str = "bearer"
    none   :str = "none"
    oauth  :str = "oauth"

    
class PluginAuthConfigOAuth(BaseModel):
    """
    The Plugin Oauth configuration as managed by JiggyBase
    Note that this is for custom plugins available in the JiggyBase enterprise tier and unrelated to the main JiggyBase plugin in the OpenAI plugin store.
    """    
    client_url:                 HttpUrl     = Field(description="ChatGPT will direct the user’s browser to this url to log in to the plugin")
    authorization_url:          HttpUrl     = Field(description="After successful login ChatGPT will complete the OAuth flow by making a POST request to this URL")
    scope:                      str         = Field(description="The scope used for the OAuth flow")
    client_id:                  str         = Field(unique=True, index=True, description="The client id to send to OpenAI for the plugin")
    client_secret:              str         = Field(description="The client secret to send to OpenAI for the plugin")
    openai_verification_token:  str         = Field(description="The verification token specified by OpenAI to configure in the plugin")

class PatchPluginOAuthConfigRequest(BaseModel):
    openai_verification_token: str  

class Source(str, Enum):
    email = "email"
    file = "file"
    chat = "chat"
    web  = "web"


class DocumentMetadata(BaseModel):
    source: Optional[Source] = None
    source_id: Optional[str] = None
    url: Optional[str] = None
    created_at: Optional[str] = None
    author: Union[str, List[str]] = None
    title: Optional[str] = None
    description: Optional[str] = None
    language: Optional[str] = Field(description="The 2 character ISO 639-1 language code of the primary language of the content.")
    version: Optional[str] = None

class DocumentChunkMetadata(DocumentMetadata):
    document_id: Optional[str] = None


class DocumentChunk(BaseModel):
    id: Optional[str] = None
    text: str
    metadata: DocumentChunkMetadata
    embedding: Optional[list[float]] = None
    token_count: Optional[int] = None
    reference_url: Optional[str] = None

    def __str__(self):
        if len(self.text) > 100:
            text = self.text[:100] + '...'
        else: 
            text = self.text
        text = text.replace('\n', ' ')
        estr = "DocumentChunk("
        if self.id is not None:
            estr += f"id={self.id}, "
        if self.metadata is not None:
            estr += f"metadata={str(self.metadata)}, "
        if self.embedding is not None:
            estr += f"embedding=dim{len(self.embedding)}, "
        estr += f"text={text})"
        return estr

    
class DocumentChunkWithScore(DocumentChunk):
    score: float


class Document(BaseModel):
    id: Optional[str]
    text: str
    metadata: Optional[DocumentMetadata] = None
    mimetype: Optional[str] = None
    token_count: Optional[int] = None


class DocumentWithChunks(Document):
    chunks: List[DocumentChunk]

class DocumentMetadataFilter(BaseModel):
    document_id: Optional[str] = None
    source: Optional[Source] = None
    source_id: Optional[str] = None
    author: Optional[str] = None
    start_date: Optional[str] = None  # any date string format
    end_date: Optional[str] = None  # any date string format
    title: Optional[str] = None


class Query(BaseModel):
    query: str
    filter: Optional[DocumentMetadataFilter] = None
    top_k: Optional[int] = 3

class QueryWithEmbedding(Query):
    embedding: list[float]


class QueryResult(BaseModel):
    query: str
    results: list[DocumentChunkWithScore]


class UpsertRequest(BaseModel):
    documents: List[Document]


class UpsertResponse(BaseModel):
    ids: List[str]


class QueryRequest(BaseModel):
    queries: List[Query]


class QueryResponse(BaseModel):
    results: List[QueryResult]


class DeleteRequest(BaseModel):
    ids: Optional[List[str]] = None
    filter: Optional[DocumentMetadataFilter] = None
    delete_all: Optional[bool] = False


class DeleteResponse(BaseModel):
    success: bool


class Accounting(BaseModel):
    chunk_count: int
    doc_count: int
    page_count: int



class DocChunksResponse(BaseModel):
    docs       : list[list[DocumentChunk]] = Field(..., description="A list of documents, each containing a list of chunks")
    next_index : int                       = Field(..., description="The index of the next document to return.  Use this value as the index parameter in the next request to get the next set of documents")  
          
